set echo on
clear screen
create table trade_iot
( timeseries_id      number not null
, reference_datetime date not null
, source_id          number not null
, volume             number
,
constraint pk_trade primary key (timeseries_id, reference_datetime, source_id)
)
--organization index
/
insert /*+ append */ into trade_iot
 select 122 + mod(trunc((level-1)/(365*5)),2)
      , trunc(sysdate) - numtodsinterval(5*mod(trunc((level-1)/5),365),'minute')
      , 342 + mod(level-1,5)
      , ceil(dbms_random.value(1,1000))
   from dual
connect by level <= 2 * 365 * 5
/
commit
/
delete trade_iot
 where (   timeseries_id = 122
       and reference_datetime = trunc(sysdate) - interval '1 00:35:00' day to second
       and source_id = 345
       )
    or (   timeseries_id in (122,123)
       and source_id in (342,343,346)
       and reference_datetime
           between trunc(sysdate) - interval '1 00:55:00' day to second
           and trunc(sysdate) - interval '1 00:35:00' day to second
       )
    or (   timeseries_id = 123
       and reference_datetime = trunc(sysdate) - interval '1 00:40:00' day to second
       )
    or (   timeseries_id in (122,123)
       and reference_datetime
           between trunc(sysdate) - interval '1 00:20:00' day to second
           and trunc(sysdate) - interval '1 00:10:00' day to second
       )
/
commit
/
exec dbms_stats.gather_table_stats(user,'trade_iot')
clear screen
select timeseries_id
     , reference_datetime
     , source_id
     , volume
  from trade_iot
 where reference_datetime
       between trunc(sysdate) - interval '1 01:00:00' day to second
       and     trunc(sysdate) - interval '1 00:00:05' day to second
 order by 1,2,3

pause
/
pause
clear screen
select trunc(sysdate) - 1 - numtodsinterval(5*level,'minute')
    from dual
 connect by level <= 12
/ 
pause
clear screen
select m.timeseries_id
     , a.datetime
     , m.source_id 
     , m.volume
  from ( select trunc(sysdate) - 1 - numtodsinterval(5*level,'minute') datetime
           from dual
        connect by level <= 12
       ) a
       left outer join trade_iot m on (a.datetime = m.reference_datetime)
 order by 1,2,3

pause
/
pause
clear screen
set autotrace on explain
select m.timeseries_id
     , a.datetime
     , m.source_id 
     , m.volume
  from ( select trunc(sysdate) - 1 - numtodsinterval(5*level,'minute') datetime
           from dual
        connect by level <= 12
       ) a
       left outer join trade_iot m
         partition by (m.timeseries_id,m.source_id)
         on (a.datetime = m.reference_datetime)
 order by 1,2,3

pause
/
set autotrace off
set echo off
drop table trade_iot purge
/
