set echo on
set feedback off
column postal_code format a11
clear screen
create table addresses
( relation_nr number(7)    not null
, address     varchar2(20) not null
, postal_code varchar2(7)  not null
, city        varchar2(20) not null
, startdate   date         not null
, constraint adr_uk unique (relation_nr,startdate)
)
/
insert into addresses values (1,'Kolkmanstraat 32','2805 CG','Gouda',to_date('01041996','ddmmyyyy'));
insert into addresses values (1,'Langerakbaan 429','3544 SP','Utrecht',to_date('01092001','ddmmyyyy'));
insert into addresses values (1,'Walmolen 1','3992 AA','Houten',to_date('01012005','ddmmyyyy'));
insert into addresses values (1,'Archimedeslaan 16','3999 ZZ','Nieuwegein',to_date('01012006','ddmmyyyy'));
insert into addresses values (2,'Nieuwe Pyramide 57','3962 HZ','Wijk bij Duurstede',to_date('01011990','ddmmyyyy'));
insert into addresses values (2,'Meerkollaan 15','4444 YY','Eindhoven',to_date('01062000','ddmmyyyy'));

commit;
exec dbms_stats.gather_table_stats(user,'addresses',cascade=>true)
pause
set echo on
set feedback on
clear screen
select * from addresses order by 1,5
/
pause
clear screen
set autotrace on explain
select relation_nr
     , address
     , postal_code
     , city
     , startdate
  from addresses a1
 where startdate = 
       ( select max(startdate)
           from addresses a2
          where a1.relation_nr = a2.relation_nr
            and a2.startdate < sysdate
       )
/
pause
clear screen
select relation_nr
     , address
     , postal_code
     , city
     , startdate
  from ( select a.*
              , lead(startdate,1,date '4712-12-31')
                over
                (partition by relation_nr order by startdate) next_startdate
           from addresses a
       )
 where sysdate between startdate and next_startdate
/
pause
clear screen
select relation_nr
     , max(address) keep (dense_rank last order by startdate) address
     , max(postal_code) keep (dense_rank last order by startdate) postal_code
     , max(city) keep (dense_rank last order by startdate) city
     , max(startdate)
  from addresses
 where startdate <= sysdate
 group by relation_nr
/
set autotrace off
pause
clear screen
select relation_nr
     , max(address) keep (dense_rank last order by startdate) address
     , min(postal_code) keep (dense_rank last order by startdate) postal_code
     , max(city) keep (dense_rank first order by startdate desc) city
     , max(startdate)
  from addresses
 where startdate <= sysdate
 group by relation_nr
/
pause
clear screen
alter table addresses drop constraint adr_uk
/
insert into addresses values (2,'Dorpstraat 1','1111 AA','Ons Dorp',to_date('01062000','ddmmyyyy'))
/
select *
  from addresses
 order by relation_nr
     , startdate
/
pause
select relation_nr
     , min(address) keep (dense_rank last order by startdate) min_address
     , max(address) keep (dense_rank last order by startdate) max_address
     , min(postal_code) keep (dense_rank last order by startdate) min_postal_code
     , max(postal_code) keep (dense_rank last order by startdate) max_postal_code
     , min(city) keep (dense_rank last order by startdate) min_city
     , max(city) keep (dense_rank last order by startdate) max_city
     , max(startdate)
  from addresses
 where startdate <= sysdate
 group by relation_nr
/
set echo off
drop table addresses purge
/
