set recsep off
set echo on
remark
remark
remark
create table bills (id,customer,total_amount)
as
select 1, 'Oracle', 1000 from dual union all
select 2, 'Google', 500 from dual union all
select 3, 'Apple', 200 from dual
/
alter table bills add constraint bills_pk primary key (id)
/
create table bill_lines (id,bill_id,product,unit_price,quantity)
as
select 1, 1, 'A' , 8, 100 from dual union all
select 2, 1, 'B', 20, 10 from dual union all
select 3, 2, 'A', 8, 50 from dual union all
select 4, 2, 'C', 100, 1 from dual union all
select 5, 3, 'A', 8, 10 from dual union all
select 6, 3, 'B', 20, 1 from dual union all
select 7, 3, 'C', 100, 1 from dual
/
create sequence bill_line_seq start with 8 increment by 1
/
alter table bill_lines
  add constraint bill_lines_pk
  primary key (id)
/
alter table bill_lines
  add constraint bill_lines_bills_fk
  foreign key (bill_id)
  references bills(id)
  on delete cascade
/
create procedure add_bill_line
( p_bill_id    in bills.id%type
, p_product    in bill_lines.product%type
, p_quantity   in bill_lines.quantity%type
, p_unit_price in bill_lines.unit_price%type
)
is
begin
  insert into bill_lines
  ( id
  , bill_id
  , product
  , quantity
  , unit_price
  )
  values
  ( bill_line_seq.nextval
  , p_bill_id
  , p_product
  , p_quantity
  , p_unit_price
  );
  update bills
     set total_amount = total_amount + p_quantity * p_unit_price
   where id = p_bill_id
  ;
end add_bill_line;
/
show err
create function sleep(p_seconds in number) return number
as
begin
  dbms_lock.sleep(p_seconds);
  return null;
end;
/
exec dbms_stats.gather_table_stats(user,'bills',cascade=>true)
exec dbms_stats.gather_table_stats(user,'bill_lines',cascade=>true)
remark
remark  To prevent bulk fetching 100 rows at once and switch to row by row processing in a FOR loop.
remark
alter session set plsql_optimize_level = 0
/
pause
create procedure p1 (p_number_of_seconds in number default 0)
as
begin
  for r_bills in
  ( select id
         , customer
         , total_amount
         , sleep(p_number_of_seconds)  -- Add some slowness to the query
      from bills
     order by customer
  )
  loop
    dbms_output.put_line
    ( 'Bill for customer '  || r_bills.customer ||
      ' of ' || to_char(r_bills.total_amount) || ' euros, consists of:'
    );
    for r_bill_lines in
    ( select product
           , unit_price
           , quantity
        from bill_lines
       where bill_id = r_bills.id
       order by product
    )
    loop
      dbms_output.put_line
      ( '* Product ' || r_bill_lines.product || ': ' || lpad(to_char(r_bill_lines.unit_price * r_bill_lines.quantity),4) ||
        '     (' || to_char(r_bill_lines.quantity) || ' * ' || r_bill_lines.unit_price || ')'
      );
    end loop;
  end loop;
end p1;
/
exec p1
pause
remark
remark  While the below block is executing,
remark  open a new session and execute:
remark
remark  exec add_bill_line(2,'R',10,100)
remark  delete bills where id = 3;
remark  commit;
remark
pause
exec p1
pause
select case
       when row_number() over (partition by b.customer order by bl.id) = 1
       then
         'Bill for customer '  || b.customer ||
         ' of ' || to_char(b.total_amount) || ' euros, consists of:' || chr(10)
       end ||
       '* Product ' || bl.product || ': ' ||
       lpad(to_char(bl.unit_price * bl.quantity),4) ||
       '     (' || to_char(bl.quantity) || ' * ' || bl.unit_price || ')' text
  from bills b
     , bill_lines bl
 where b.id = bl.bill_id
 order by b.customer
     , bl.product
/
pause
select case grouping(bl.id)
       when 0 then
         '* Product ' || bl.product || ': ' ||
         lpad(to_char(bl.unit_price * bl.quantity),4) ||
         '     (' || to_char(bl.quantity) || ' * ' || bl.unit_price || ')'
       when 1 then
         'Bill for customer '  || b.customer ||
         ' of ' || to_char(b.total_amount) || ' euros, consists of:'
       end text
  from bills b
     , bill_lines bl
 where b.id = bl.bill_id
 group by b.id
     , b.customer
     , b.total_amount
     , rollup((bl.id,bl.product,bl.quantity,bl.unit_price))
 order by b.customer
     , grouping(bl.id) desc
     , bl.product
/
pause
select text
  from bills b
     , bill_lines bl
 where b.id = bl.bill_id
 model
       partition by (b.customer,b.total_amount)
       dimension by (bl.id)
       measures (bl.product,bl.unit_price,bl.quantity,cast(null as varchar2(100)) text)
       ( text[any]
         = '* Product ' || product[cv()] || ': ' ||
           lpad(to_char(unit_price[cv()] * quantity[cv()]),4) ||
           '     (' || to_char(quantity[cv()]) || ' * ' || unit_price[cv()] || ')'
       , text[0]
         = 'Bill for customer ' || cv(customer) ||
           ' of ' || to_char(cv(total_amount)) || ' euros, consists of:'
       )
 order by customer
     , product nulls first
/
set echo off
drop table bill_lines purge
/
drop table bills purge
/
drop procedure add_bill_line
/
drop function sleep
/
drop procedure p1
/
drop sequence bill_line_seq
/
set recsep wrap