set echo on
clear screen
create table t (grp,startdate,enddate,value)
as
select 'grp1', date '2008-01-01', date '2008-01-10', 11 from dual union all
select 'grp2', date '2008-01-01', date '2008-01-01', 22 from dual union all
select 'grp3', date '2008-01-01', date '2008-01-03', 33 from dual
/
exec dbms_stats.gather_table_stats(user,'t')
select * from t
/
pause
set serveroutput off
clear screen
remark
remark  Cartesian product
remark
select t.grp
     , t.startdate + l - 1 as day
     , t.value
  from t
     , (select level l from dual connect by level <= 1000)
 where l <= t.enddate - t.startdate + 1
 order by t.grp
     , day
/
pause
break on grp skip 1
select /*+ gather_plan_statistics */ 
       t.grp
     , t.startdate + l - 1 as day
     , t.value
  from t
     , (select level l from dual connect by level <= 1000)
 where l <= t.enddate - t.startdate + 1
 order by t.grp
     , day
/
pause
select * from table(dbms_xplan.display_cursor(null,null,'ALLSTATS LAST'))
/
pause
clear screen
remark
remark  Cartesian product with calculated upperbound
remark
select /*+ gather_plan_statistics */ 
       m.grp
     , m.startdate + l as day
     , m.value
  from t m
     , ( select level-1 l
           from (select max(enddate-startdate+1) maxinterval from t)
        connect by level <= maxinterval
       )
 where l <= m.enddate - m.startdate
 order by m.grp
     , day
/
pause
select * from table(dbms_xplan.display_cursor(null,null,'ALLSTATS LAST'))
/
pause
clear screen
remark
remark  Multiset
remark
select /*+ gather_plan_statistics */ 
       t.grp
     , t.startdate + (row_number() over (partition by t.grp order by null)-1) as day
     , t.value
  from t
     , table
       ( cast
         ( multiset(select null from dual connect by rownum <= (t.enddate-t.startdate)+1)
           as sys.dbms_debug_vc2coll
         )
       ) x
 order by t.grp
     , day
/
pause
set linesize 200
select * from table(dbms_xplan.display_cursor(null,null,'ALLSTATS -MEMSTATS LAST'))
/
set linesize 135
pause
clear screen
remark
remark  Model clause
remark
select /*+ gather_plan_statistics */ 
       grp
     , day
     , value
  from t
 model
       partition by (grp,value)
       dimension by (0 i)
       measures (startdate day, enddate)
       rules
       ( day[for i from 1 to enddate[0] - day[0] increment 1] = day[0] + cv(i)
       )
 order by grp
     , day
/
pause
select * from table(dbms_xplan.display_cursor(null,null,'ALLSTATS LAST'))
/
pause
clear screen
remark
remark  Recursive Subquery Factoring
remark
with tr (grp,day,value,enddate) as
( select grp
       , startdate
       , value
       , enddate
    from t
   union all
  select tr.grp
       , tr.day + 1
       , tr.value
       , tr.enddate
    from tr
   where tr.day < tr.enddate
)
select /*+ gather_plan_statistics */
       grp
     , day
     , value
  from tr
/
pause
select * from table(dbms_xplan.display_cursor(null,null,'ALLSTATS LAST'))
/
set echo off
drop table t purge
/
clear breaks
set serveroutput on
