set echo on
set linesize 4000
clear screen
declare
  l_xml_as_clob clob;
begin
  select xmlserialize
         ( document xmlroot
           ( xmlelement
             ( "Departments"
             , xmlagg
               ( xmlelement
                 ( "Department"
                 , xmlforest
                   ( deptno as "Number"
                   , dname as "Name"
                   )
                 )
               )
             )
           ) as clob indent
         )
    into l_xml_as_clob
    from dept
   where deptno in (10,20)
  ;
  dbms_output.put_line(l_xml_as_clob);
end;
/
pause
clear screen
declare
  l_xml            xmltype;
  l_context_handle dbms_xmlgen.ctxHandle;
begin
  l_context_handle := dbms_xmlgen.newContext
  ( 'select d.deptno "Number"
          , d.dname "Name"
       from dept d
      where d.deptno in (10,20)'
  );
  dbms_xmlgen.setRowSetTag(l_context_handle, 'Departments');
  dbms_xmlgen.setRowTag(l_context_handle, 'Department');
  l_xml := xmltype(dbms_xmlgen.getXML(l_context_handle));
  dbms_xmlgen.closeContext(l_context_handle);
  dbms_output.put_line(l_xml.getClobVal());
end;
/
pause
clear screen
declare
  l_rc  sys_refcursor;
  l_xml xmltype;
begin
  open l_rc
  for
  select deptno "Number"
       , dname "Name"
    from dept
   where deptno in (10,20)
  ;
  l_xml := xmltype(l_rc);
  close l_rc;
  dbms_output.put_line(l_xml.getClobVal());
end;
/
pause
clear screen
declare
  l_rc  sys_refcursor;
  l_xml xmltype;
begin
  open l_rc
  for
  select deptno "Number"
       , dname "Name"
    from dept
   where deptno in (10,20)
  ;
  l_xml := xmltype(l_rc);
  close l_rc;
  dbms_output.put_line
  ( replace
    ( replace
      ( replace
        ( replace
          ( l_xml.getClobVal()
          , '<ROWSET>'
          , '<Departments>'
          )
        , '</ROWSET>'
        , '</Departments>'
        )
      , '<ROW>'
      , '<Department>'
      )
    , '</ROW>'
    , '</Department>'
    )
  );
end;
/
pause
clear screen
declare
  l_rc  sys_refcursor;
  l_xml xmltype;
begin
  select xmlroot
         ( xmltransform
           ( xmltype
             ( cursor
               ( select d.deptno
                      , d.dname
                   from dept d
                  where d.deptno in (10,20)
               )
             )
           , xmltype.createxml
             ( '<?xml version="1.0"?>
                <xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" >
                <xsl:template match="/">
                <Departments>
                <xsl:for-each select="/ROWSET/ROW">
                  <Department>
                  <Number><xsl:value-of select="DEPTNO"/></Number>
                  <Name><xsl:value-of select="DNAME"/></Name>
                  </Department>
                </xsl:for-each>
                </Departments>
                </xsl:template>
                </xsl:stylesheet>'
             )
           )
         )
    into l_xml
    from dual
  ;
  dbms_output.put_line(l_xml.getClobVal());
end;
/
pause
clear screen
declare
  l_xml_as_clob clob;
begin
  select xmlserialize
         ( document xmlroot
           ( xmlelement
             ( "Departments"
             , xmlagg
               ( xmlelement
                 ( "Department"
                 , xmlforest
                   ( d.deptno as "Number"
                   , d.dname as "Name"
                   )
                 , xmlelement
                   ( "Employees"
                   , xmlagg
                     ( xmlelement
                       ( "Employee"
                       , xmlforest
                         ( e.ename as "Name"
                         , e.sal as "Salary"
                         )
                       )
                     )
                   )
                 )
               )
             )
           ) as clob indent
         )
    into l_xml_as_clob
    from dept d
       , emp e
   where d.deptno = e.deptno
     and d.deptno in (10,20)
   group by d.deptno
       , d.dname
  ;
  dbms_output.put_line(l_xml_as_clob);
end;
/
pause
clear screen
declare
  l_xml            xmltype;
  l_context_handle dbms_xmlgen.ctxHandle;
begin
  l_context_handle := dbms_xmlgen.newContext
  ( 'select d.deptno "Number"
          , d.dname "Name"
          , cursor
            ( select e.ename "Name"
                   , e.sal "Salary"
                from emp e
               where e.deptno = d.deptno
            ) "Employees"
       from dept d
      where d.deptno in (10,20)'
  );
  dbms_xmlgen.setRowSetTag(l_context_handle, 'Departments');
  dbms_xmlgen.setRowTag(l_context_handle, 'Department');
  l_xml := xmltype(dbms_xmlgen.getXML(l_context_handle));
  dbms_xmlgen.closeContext(l_context_handle);
  dbms_output.put_line
  ( replace
    ( replace
      ( l_xml.getClobVal()
      , '<Employees_ROW>'
      , '<Employee>'
      )
    , '</Employees_ROW>'
    , '</Employee>'
    )
  );
end;
/
pause
clear screen
declare
  l_rc  sys_refcursor;
  l_xml xmltype;
begin
  open l_rc
  for
  select d.deptno "Number"
       , d.dname "Name"
       , cursor
         ( select e.ename "Name"
                , e.sal "Salary"
             from emp e
            where e.deptno = d.deptno
         ) "Employees"
    from dept d
   where d.deptno in (10,20)
  ;
  l_xml := xmltype(l_rc);
  close l_rc;
  dbms_output.put_line
  ( replace
    ( replace
      ( replace
        ( replace
          ( replace
            ( replace
              ( l_xml.getClobVal()
              , '<Employees_ROW>'
              , '<Employee>'
              )
            , '</Employees_ROW>'
            , '</Employee>'
            )
          , '<ROWSET>'
          , '<Departments>'
          )
        , '</ROWSET>'
        , '</Departments>'
        )
      , '<ROW>'
      , '<Department>'
      )
    , '</ROW>'
    , '</Department>'
    )
  );
end;
/
pause
clear screen
declare
  l_rc  sys_refcursor;
  l_xml xmltype;
begin
  open l_rc
  for
  select d.deptno
       , d.dname
       , cursor
         ( select e.ename
                , e.sal
             from emp e
            where e.deptno = d.deptno
         ) emps
    from dept d
   where d.deptno in (10,20)
  ;
  l_xml := xmltype(l_rc);
  close l_rc
  ;
  select xmlroot
         ( xmltransform
           ( l_xml
           , xmltype
             ( '<?xml version="1.0"?>
                <xsl:stylesheet version="1.0"
                  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" >
                <xsl:template match="/">
                <Departments>
                <xsl:for-each select="/ROWSET/ROW">
                  <Department>
                    <Number><xsl:value-of select="DEPTNO"/></Number>
                    <Name><xsl:value-of select="DNAME"/></Name>
                    <Employees>
                    <xsl:for-each select="EMPS/EMPS_ROW">
                      <Employee>
                        <Name><xsl:value-of select="ENAME"/></Name>
                        <Salary><xsl:value-of select="SAL"/></Salary>
                      </Employee>
                    </xsl:for-each>
                    </Employees>
                  </Department>
                </xsl:for-each>
                </Departments>
                </xsl:template>
                </xsl:stylesheet>'
             )
           )
         )
    into l_xml
    from dual
  ;
  dbms_output.put_line(l_xml.getClobVal());
end;
/
set echo off
set linesize 135
