set linesize 80
column name format a16
column creation_time format a24
set echo on
clear screen
create table xml_documents_to_process (xml xmltype)
/
pause
declare
  l xmltype;
begin
  l := xmltype
  ( '<?xml version="1.0" encoding="UTF-8"?>
<msg:message xmlns:msg="namespace1">
  <msg:metadata>
    <msg:CreationTime>2010-03-16T15:23:14.56</msg:CreationTime>
  </msg:metadata>
  <msg:content xmlns:tcim="namespace2" >
    <tcim:ReportingDate>2010-03-19</tcim:ReportingDate>
    <tcim:Persons>
      <tcim:Person>
        <tcim:Name>Jeffrey Lebowski</tcim:Name>
        <tcim:Items>
          <tcim:Item>
            <tcim:Name>Rug</tcim:Name>
            <tcim:Remarks>Really ties the room together</tcim:Remarks>
          </tcim:Item>
          <tcim:Item>
            <tcim:Name>Creedence Tapes</tcim:Name>
            <tcim:Remarks>Stolen</tcim:Remarks>
          </tcim:Item>
          <tcim:Item>
            <tcim:Name>Toe</tcim:Name>
            <tcim:Remarks>With nail polish</tcim:Remarks>
          </tcim:Item>
        </tcim:Items>
      </tcim:Person>
      <tcim:Person>
        <tcim:Name>Jesus Quintana</tcim:Name>
        <tcim:Items>
          <tcim:Item>
            <tcim:Name>Bowling ball</tcim:Name>
            <tcim:Remarks>Warmed up</tcim:Remarks>
          </tcim:Item>
        </tcim:Items>
      </tcim:Person>
      <tcim:Person>
        <tcim:Name>Walter Sobchak</tcim:Name>
      </tcim:Person>
    </tcim:Persons>
  </msg:content>
</msg:message>'
  );
  insert into xml_documents_to_process values (l);
end;
/
pause
create table messages
( guid           raw(16)      primary key
, creation_time  timestamp(2) not null
, reporting_date date         not null
)
/
create table persons
( guid         raw(16)      primary key
, message_guid raw(16)      not null references messages(guid)
, name         varchar2(30) not null
)
/
create table items
( guid        raw(16)      primary key
, person_guid raw(16)      not null references persons(guid)
, name        varchar2(20) not null
, remarks     varchar2(40) not null
)
/
pause
create procedure from_xml_to_relational (p_xml in xmltype)
is
begin
  insert all
    when add_messages_indicator = 'Y'
    then
         into messages values
         ( m_guid
         , creation_time
         , reporting_date
         )
    when add_persons_indicator = 'Y'
    then
         into persons values
         ( p_guid
         , m_guid
         , p_name
         )
    when add_items_indicator = 'Y'
    then
         into items values
         ( i_guid
         , p_guid
         , i_name
         , remarks
         )
  select case when p.id = 1 and nvl(i.id,1) = 1 then 'Y' end add_messages_indicator
       , case when nvl(i.id,1) = 1 then 'Y' end              add_persons_indicator
       , nvl2(i.id,'Y',null)                                 add_items_indicator
       , first_value(sys_guid()) over (partition by m.id order by p.id,i.id) m_guid
       , first_value(sys_guid()) over (partition by m.id,p.id order by i.id) p_guid
       , sys_guid()                                                          i_guid
       , to_timestamp(m.creation_time,'yyyy-mm-dd"T"hh24:mi:ss.ff') creation_time
       , to_date(m.reporting_date,'yyyy-mm-dd')                     reporting_date
       , p.name                                                     p_name
       , i.name                                                     i_name
       , i.remarks                                                  remarks
    from xmltable
         ( xmlnamespaces ('namespace1' as "msg", default 'namespace2')
         , 'msg:message'
           passing p_xml
           columns id                          for ordinality
                 , creation_time  varchar2(50) path 'msg:metadata/msg:CreationTime'
                 , reporting_date varchar2(50) path 'msg:content/ReportingDate'
                 , persons        xmltype      path 'msg:content/Persons'
         ) m 
       , xmltable
         ( xmlnamespaces (default 'namespace2')
         , 'Persons/Person'
           passing m.persons 
           columns id                 for ordinality
                 , name  varchar2(30) path 'Name'
                 , items xmltype      path 'Items'
         ) p
       , xmltable
         ( xmlnamespaces(default 'namespace2')
         , 'Items/Item'
           passing p.items
           columns id                   for ordinality
                 , name    varchar2(20) path 'Name'
                 , remarks varchar2(40) path 'Remarks'
         ) (+) i 
  ;
end;
/
pause
declare
  l_xml xmltype;
begin
  select xml
    into l_xml
    from xml_documents_to_process
  ;
  from_xml_to_relational(l_xml);
end;
/
pause
select * from messages
/
pause
select * from persons
/
pause
select * from items
/
pause
select m.creation_time
     , m.reporting_date
     , p.name
     , i.name
     , i.remarks
  from messages m
     , persons p
     , items i
 where m.guid = p.message_guid
   and p.guid = i.person_guid (+)
/
set echo off
drop table xml_documents_to_process purge
/
drop table items purge
/
drop table persons purge
/
drop table messages purge
/
drop procedure from_xml_to_relational
/
set linesize 134
